﻿#include "Waffle.h"
#include <QPainter>
#include <QPaintEvent>

Waffle::Waffle(int flag, QWidget *parent) : QWidget(parent) {
    // //根据不同窗口设置不同大小 0：小 1：中 2：大
    // if (flag == 0) {
    //     // 华夫台的参数
    //     gridSize = 5;  // 每个格子的大小
    //     gap = 3;        // 格子之间的间隙
    //     boxGap = 6;    // 华夫台之间的间隙
    // } else if (flag == 1) {
    //     // 华夫台的参数
    //     gridSize = 20;  // 每个格子的大小
    //     gap = 6;        // 格子之间的间隙
    //     boxGap = 12;    // 华夫台之间的间隙
    // } else if (flag == 2) {
    //     // 华夫台的参数
    //     gridSize = 50;  // 每个格子的大小
    //     gap = 6;        // 格子之间的间隙
    //     boxGap = 12;    // 华夫台之间的间隙
    // }
    rows = 6;
    cols = 18;
    centerX = 25;
    centerY = 25;
    // 随机数初始化
    std::srand(std::time(nullptr)); // 使用当前时间作为种子
    for (int row = 0; row < rows; ++row) {
        for (int col = 0; col < cols; ++col) {
            WAFFLE_MATRIX_POINT_STRUCT point;
            point.nPackMatrixId = 1;
            point.nPackRow = 6;
            point.nPackCol = 18;
            point.nDieRow = row;
            point.nDieCol = col;
            point.nDieMatrixId = 1;
            point.iDieIndex = 1;
            point.bDisable = false; // 默认为可用
            point.stPosition.x = col * 10.0;
            point.stPosition.y = row * 10.0;
            int randomStatus = std::rand() % 5; // 随机选取DIE_EXIST, PICK_ING, NO_EXIST, SKIP_DIE
            point.eStatus = static_cast<PICK_DIE_STATUS>(randomStatus);
            // 将点添加到waferData容器中
            waffleData.append(point);
        }
    }

}

void Waffle::paintEvent(QPaintEvent *event) {
    QPainter painter(this);

    // 设置抗锯齿，使绘图更平滑
    painter.setRenderHint(QPainter::Antialiasing);

    // 绘制背景
    painter.fillRect(this->rect(), Qt::white);


    // 计算每个华夫台的总宽度和高度
    int totalWidth = cols * gridSize + (cols - 1) * gap;
    int totalHeight = rows * gridSize + (rows - 1) * gap;

    // 计算华夫盒的总宽度和高度
    int boxTotalWidth = boxCols * totalWidth + (boxCols - 1) * boxGap;
    int boxTotalHeight = boxRows * totalHeight + (boxRows - 1) * boxGap;

    // 确定华夫盒的起始位置，使其居中
    int boxStartX = (width() - boxTotalWidth) / 2;
    int boxStartY = (height() - boxTotalHeight) / 2;

    // 绘制多个华夫台
    for (int i = 0; i < boxRows; ++i) {
        for (int j = 0; j < boxCols; ++j) {
            // 计算每个华夫台的起始位置
            int startX = boxStartX + j * (totalWidth + boxGap);
            int startY = boxStartY + i * (totalHeight + boxGap);

            // 绘制单个华夫台
            drawWaffle(painter, startX, startY, rows, cols, gridSize, gap);
        }
    }
}

void Waffle::drawWaffle(QPainter &painter, int startX, int startY, int rows, int cols, int gridSize, int gap) {
    // 绘制网格
    for (int i = 0; i < rows; ++i) {
        for (int j = 0; j < cols; ++j) {
            // 计算每个格子的坐标
            int x = startX + j * (gridSize + gap);
            int y = startY + i * (gridSize + gap);
            QRect cellRect(x, y, gridSize, gridSize);

            // 填充颜色
            painter.fillRect(cellRect, Qt::yellow);

            // 绘制边框
            painter.setPen(Qt::black);
            painter.drawRect(cellRect);
        }
    }
}
QColor Waffle::getColorByStatus(PICK_DIE_STATUS status) {
    switch (status) {
    case DIE_EXIST: return QColor(0, 102, 255); // 蓝色
    case NO_EXIST: return QColor(200, 200, 200); // 浅灰
    case PICK_ING: return QColor(255, 255, 0); // 黄色
    case SKIP_DIE: return QColor(128, 128, 128); // 深灰
    case EDGE_DIE: return QColor(255, 165, 0); // 橙色
    default: return QColor(0, 0, 0); // 默认黑色
    }
}
void Waffle::paintInitFrom(QWidget *parent) {

    // 获取当前窗口的宽高
    int width = parent->width();
    int height = parent->height();

    // 根据行列数计算每个晶圆点的大小，选择小的边来决定
    int cellSize = qMin(width, height) / qMax(rows, cols); // 固定大小为正方形，按最小边计算

    // 计算左上角偏移量，居中显示
    int offsetX = (width - cellSize * cols) / 2;
    int offsetY = (height - cellSize * rows) / 2;

    // 创建一个 QPixmap 对象用于保存绘制的图像
    globalPixmap = QPixmap(width, height);
    globalPixmap.fill(Qt::white);  // 填充背景色为白色

    // 创建 QPainter 以绘制到 QPixmap 上
    QPainter painter(&globalPixmap);
    painter.setRenderHint(QPainter::Antialiasing); // 启用抗锯齿

    // 设置画笔为无边框
    painter.setPen(Qt::NoPen);

    // 绘制每个晶圆点
    for (int i = 0; i < waffleData.size(); ++i) {
        int x = offsetX + waffleData[i].nDieCol * cellSize;
        int y = offsetY + waffleData[i].nDieRow * cellSize;

        // 根据点的状态设置颜色
        painter.setBrush(getColorByStatus(waffleData[i].eStatus));

        // 绘制晶圆点（每个点是一个矩形）
        painter.drawRect(x, y, cellSize, cellSize);
    }


    // painter.setBrush(Qt::black);
    // painter.drawEllipse(centerX - 3, centerY - 3, 6, 6);
    painter.end(); // 结束绘制
}
void Waffle::initFrom(QWidget *parent) {
    scene = new QGraphicsScene(parent);
    view = new WaffleGraphicsView(scene);

    // 获取当前窗口的宽高
    int width = parent->width();
    int height = parent->height();

    // 根据行列数计算每个晶圆点的大小，选择小的边来决定
    int cellSize = qMin(width, height) / qMax(rows, cols); // 固定大小为正方形，按最小边计算

    for (int i = 0; i < waffleData.size(); ++i) {
        DieItem* die = new DieItem(waffleData[i].nDieRow, waffleData[i].nDieCol, waffleData[i].eStatus, cellSize);
        die->setPos(waffleData[i].nDieCol * cellSize, waffleData[i].nDieRow * cellSize);
        scene->addItem(die);
    }

    view->setSceneRect(scene->itemsBoundingRect());
    view->resize(width, height);
}
QPixmap Waffle::getGlobalPixmap() const {
    return globalPixmap;
}
